const defaultRoutes = {
    upsert: ()=>({
            path: "/bundles"
        }),
    list: ()=>({
            path: "/bundles",
            headers: {
                "Cache-Control": "no-cache"
            }
        }),
    retrieve: (bundleId)=>({
            path: `/bundles/${bundleId}`,
            headers: {
                Accept: "application/json"
            }
        })
};
const createRoute = (defaultRoute, customRoute)=>({
        path: customRoute?.path ?? defaultRoute.path,
        headers: {
            ...defaultRoute.headers,
            ...customRoute?.headers
        }
    });
const standaloneRepository = (config, hooks)=>(_)=>{
        const routes = {
            upsert: ()=>createRoute(defaultRoutes.upsert(), config.routes?.upsert?.()),
            list: ()=>createRoute(defaultRoutes.list(), config.routes?.list?.()),
            retrieve: (bundleId)=>createRoute(defaultRoutes.retrieve(bundleId), config.routes?.retrieve?.(bundleId))
        };
        const getHeaders = (routeHeaders)=>({
                "Content-Type": "application/json",
                ...config.commonHeaders,
                ...routeHeaders
            });
        let bundles = [];
        const changedIds = new Set();
        function markChanged(id) {
            changedIds.add(id);
        }
        return {
            name: "standalone-repository",
            async commitBundle () {
                if (0 === changedIds.size) return;
                const changedBundles = bundles.filter((b)=>changedIds.has(b.id));
                if (0 === changedBundles.length) return;
                const { path, headers: routeHeaders } = routes.upsert();
                const response = await fetch(`${config.baseUrl}${path}`, {
                    method: "POST",
                    headers: getHeaders(routeHeaders),
                    body: JSON.stringify(changedBundles)
                });
                if (!response.ok) throw new Error(`API Error: ${response.statusText}`);
                const result = await response.json();
                if (!result.success) throw new Error("Failed to commit bundles");
                changedIds.clear();
                hooks?.onDatabaseUpdated?.();
            },
            async updateBundle (targetBundleId, newBundle) {
                bundles = await this.getBundles();
                const targetIndex = bundles.findIndex((u)=>u.id === targetBundleId);
                if (-1 === targetIndex) throw new Error("target bundle version not found");
                Object.assign(bundles[targetIndex], newBundle);
                markChanged(targetBundleId);
            },
            async appendBundle (inputBundle) {
                bundles = await this.getBundles();
                bundles.unshift(inputBundle);
                markChanged(inputBundle.id);
            },
            async getBundleById (bundleId) {
                try {
                    const { path, headers: routeHeaders } = routes.retrieve(bundleId);
                    const response = await fetch(`${config.baseUrl}${path}`, {
                        method: "GET",
                        headers: getHeaders(routeHeaders)
                    });
                    if (!response.ok) return null;
                    return await response.json();
                } catch (error) {
                    return null;
                }
            },
            async getBundles (refresh = false) {
                if (bundles.length > 0 && !refresh) return bundles;
                const { path, headers: routeHeaders } = routes.list();
                const response = await fetch(`${config.baseUrl}${path}`, {
                    method: "GET",
                    headers: getHeaders(routeHeaders)
                });
                if (!response.ok) throw new Error(`API Error: ${response.statusText}`);
                bundles = await response.json();
                return bundles;
            }
        };
    };
export { standaloneRepository };
